diff --git a/src/backend/optimizer/plan/createplan.c b/src/backend/optimizer/plan/createplan.c
index 762dfb1..5f88466 100644
*** a/src/backend/optimizer/plan/createplan.c
--- b/src/backend/optimizer/plan/createplan.c
***************
*** 27,32 ****
--- 27,33 ----
  #include "optimizer/plancat.h"
  #include "optimizer/planmain.h"
  #include "optimizer/predtest.h"
+ #include "optimizer/prep.h"
  #include "optimizer/restrictinfo.h"
  #include "optimizer/tlist.h"
  #include "optimizer/var.h"
*************** create_append_plan(PlannerInfo *root, Ap
*** 546,551 ****
--- 547,553 ----
  	List	   *tlist = build_relation_tlist(best_path->path.parent);
  	List	   *subplans = NIL;
  	ListCell   *subpaths;
+ 	bool	    can_optimize_minmax;
  
  	/*
  	 * It is possible for the subplans list to contain only one entry, or even
*************** create_append_plan(PlannerInfo *root, Ap
*** 566,577 ****
  									NULL);
  	}
  
  	/* Normal case with multiple subpaths */
  	foreach(subpaths, best_path->subpaths)
  	{
  		Path	   *subpath = (Path *) lfirst(subpaths);
  
! 		subplans = lappend(subplans, create_plan(root, subpath));
  	}
  
  	plan = make_append(subplans, false, tlist);
--- 568,618 ----
  									NULL);
  	}
  
+ 	can_optimize_minmax = can_optimize_append_minmax(root);
+ 
  	/* Normal case with multiple subpaths */
  	foreach(subpaths, best_path->subpaths)
  	{
  		Path	   *subpath = (Path *) lfirst(subpaths);
  
! 		RelOptInfo		*parentrel = best_path->path.parent;
! 		RelOptInfo		*childrel = subpath->parent;
! 		AppendRelInfo	*appinfo = NULL;
! 		ListCell		*lc;
! 		Plan			*subplan = NULL;
! 
! 		if (!can_optimize_minmax)
! 		{
! 			subplans = lappend(subplans, create_plan(root, subpath));
! 			continue;
! 		}
! 
! 		/* only try to optimize children rel's */
! 		foreach (lc, root->append_rel_list)
! 		{
! 			AppendRelInfo *a = (AppendRelInfo *) lfirst(lc);
! 			
! 			if (a->child_relid == childrel->relid &&
! 				a->parent_relid == parentrel->relid)
! 			{
! 				appinfo = a;
! 				break;
! 			}
! 		}
! 
! 		if (appinfo)
! 		{
! 			List *targetlist =
! 				(List *) adjust_appendrel_attrs((Node *) root->parse->targetList,
! 												appinfo);
! 			subplan = do_optimize_minmax_aggregates(root, targetlist, childrel,
! 													subpath);
! 		}
! 
! 		if (subplan)
! 			subplans = lappend(subplans, subplan);
! 		else
! 			subplans = lappend(subplans, create_plan(root, subpath));
  	}
  
  	plan = make_append(subplans, false, tlist);
diff --git a/src/backend/optimizer/plan/planagg.c b/src/backend/optimizer/plan/planagg.c
index 34e9617..959cc2c 100644
*** a/src/backend/optimizer/plan/planagg.c
--- b/src/backend/optimizer/plan/planagg.c
*************** static void make_agg_subplan(PlannerInfo
*** 53,59 ****
  static Node *replace_aggs_with_params_mutator(Node *node, List **context);
  static Oid	fetch_agg_sort_op(Oid aggfnoid);
  
- 
  /*
   * optimize_minmax_aggregates - check for optimizing MIN/MAX via indexes
   *
--- 53,58 ----
*************** optimize_minmax_aggregates(PlannerInfo *
*** 77,89 ****
  	RangeTblRef *rtr;
  	RangeTblEntry *rte;
  	RelOptInfo *rel;
- 	List	   *aggs_list;
- 	ListCell   *l;
- 	Cost		total_cost;
- 	Path		agg_p;
- 	Plan	   *plan;
- 	Node	   *hqual;
- 	QualCost	tlist_cost;
  
  	/* Nothing to do if query has no aggregates */
  	if (!parse->hasAggs)
--- 76,81 ----
*************** optimize_minmax_aggregates(PlannerInfo *
*** 124,129 ****
--- 116,137 ----
  		return NULL;
  	rel = find_base_rel(root, rtr->rtindex);
  
+ 	return do_optimize_minmax_aggregates(root, tlist, rel, best_path);
+ }
+ 
+ Plan *
+ do_optimize_minmax_aggregates(PlannerInfo *root, List *tlist, RelOptInfo *rel,
+ 							  Path *best_path)
+ {
+ 	Query	   *parse = root->parse;
+ 	List	   *aggs_list;
+ 	ListCell   *l;
+ 	Cost		total_cost;
+ 	Path		agg_p;
+ 	Plan	   *plan;
+ 	Node	   *hqual;
+ 	QualCost	tlist_cost;
+ 
  	/*
  	 * Since this optimization is not applicable all that often, we want to
  	 * fall out before doing very much work if possible.  Therefore we do the
*************** optimize_minmax_aggregates(PlannerInfo *
*** 214,219 ****
--- 222,273 ----
  }
  
  /*
+  * can_optimize_append_minmax - check whether optimizing MIN/MAX via indexes
+  * can potentially be pushed down to child relations.
+  *
+  * This checks whether the query is of the form:
+  * 	1. SELECT MAX(col) FROM tab WHERE ...;
+  * 	2. SELECT MIN(col) FROM tab WHERE ...;
+  */
+ bool
+ can_optimize_append_minmax(PlannerInfo *root)
+ {
+ 	Query	   *parse = root->parse;
+ 	List	   *tlist = parse->targetList;
+ 	List	   *aggs_list;
+ 
+ 	/* Nothing to do if query has no aggregates */
+ 	if (!parse->hasAggs)
+ 		return false;
+ 
+ 	Assert(!parse->setOperations);		/* shouldn't get here if a setop */
+ 	Assert(parse->rowMarks == NIL);		/* nor if FOR UPDATE */
+ 
+ 	/*
+ 	 * Reject unoptimizable cases.
+ 	 *
+ 	 * We don't handle GROUP BY or windowing, because our current
+ 	 * implementations of grouping require looking at all the rows anyway, and
+ 	 * so there's not much point in optimizing MIN/MAX.
+ 	 */
+ 	if (parse->groupClause || parse->hasWindowFuncs)
+ 		return false;
+ 
+ 	/* We're only supporting a single MIN/MAX in the targetlist. */
+ 	if (list_length(tlist) > 1 || parse->havingQual)
+ 		return false;
+ 
+ 	aggs_list = NIL;
+ 	if (find_minmax_aggs_walker((Node *) tlist, &aggs_list))
+ 		return false;
+ 
+ 	if (list_length(aggs_list) > 1)
+ 		return false;
+ 
+ 	return true;
+ }
+ 
+ /*
   * find_minmax_aggs_walker
   *		Recursively scan the Aggref nodes in an expression tree, and check
   *		that each one is a MIN/MAX aggregate.  If so, build a list of the
diff --git a/src/include/optimizer/planmain.h b/src/include/optimizer/planmain.h
index 0dd2bcc..939c1ef 100644
*** a/src/include/optimizer/planmain.h
--- b/src/include/optimizer/planmain.h
*************** extern void query_planner(PlannerInfo *r
*** 34,39 ****
--- 34,42 ----
   */
  extern Plan *optimize_minmax_aggregates(PlannerInfo *root, List *tlist,
  						   Path *best_path);
+ extern Plan *do_optimize_minmax_aggregates(PlannerInfo *root, List *tlist,
+ 										   RelOptInfo *rel, Path *best_path);
+ extern bool can_optimize_append_minmax(PlannerInfo *root);
  
  /*
   * prototypes for plan/createplan.c
